@@ -35,12 +35,17 @@ class TemplateHTMLRenderer(TemplateHTMLRenderer):
                         link_next=link_next, link_prev=link_prev)
 
             if getattr(ser, '_errors', {}):
+                # CAUTION: bad hacks start here. This should be removed at some point
+
                 # unmark exception from response because this was a validation error
                 # This will allow TemplateHTMLRenderer to still render the template as if it was without problems
                 # If this is not done, the only result user will see will be a 404 error without any details
                 response = renderer_context['response']
                 response.exception = False
 
+                # result data should be object data otherwise nothing will render...
+                data['data'] = data['serializer'].data
+
         return super().render(data, accepted_media_type, renderer_context)
 
     def get_template_names(self, response, view):
