@@ -1,27 +1,53 @@
-from django.db import models
+from typing import Any
 
+from django.db import models
 from rest_framework import serializers
 
-from dynamicforms.action import ActionControls
-from dynamicforms.buttons import FormButtons, Button
+from dynamicforms.action import Actions
 from dynamicforms.settings import DYNAMICFORMS
 from . import fields
-from .mixins import UUIDMixIn, ActionMixin, RenderToTableMixin
+from .mixins import ActionMixin, RenderMixin
+from .struct import StructDefault
 
 
-class DynamicFormsSerializer(UUIDMixIn, ActionMixin, RenderToTableMixin):
-
+class DynamicFormsSerializer(RenderMixin, ActionMixin):
     template_name = DYNAMICFORMS.form_base_template  #: template filename for single record view (HTMLFormRenderer)
-    controls = ActionControls(add_default_crud=True)
+    actions = Actions(add_default_crud=True, add_form_buttons=True)
     form_titles = {
         'table': '',
         'new': '',
         'edit': '',
     }
-    form_buttons = FormButtons(add_default_cancel=True, add_default_save=True)
 
     show_filter = False  # When true, filter row is shown for list view
-    serializer_type = None  # Current types: None, 'filter'
+
+    def __init__(self, *args, is_filter: bool = False, **kwds):
+        self.is_filter = is_filter
+        if self.is_filter:
+            try:
+                instance = self.Meta.model()
+                for fld in instance._meta.fields:
+                    setattr(instance, fld.name, None)
+            except:
+                instance = StructDefault(_default_=None)
+            kwds.setdefault('instance', instance)
+        super().__init__(*args, **kwds)
+        try:
+            # hide the primary key field (DRF only marks it as R/O)
+            field_name = self.Meta.model._meta.pk.name
+            if field_name not in self._declared_fields:
+                pk_field = self.fields[field_name]
+                pk_field.display_form = fields.DisplayMode.HIDDEN
+                pk_field.display_table = fields.DisplayMode.FULL
+        except:
+            pass
+        if self.is_filter:
+            for field in self.fields.values():
+                field.default = None
+                field.allow_blank = True
+                field.allow_null = True
+                field.read_only = False
+                field.display_form = field.display_table  # filter's form is same as non-filter's table
 
     @property
     def has_non_field_errors(self):
@@ -55,21 +81,10 @@ class DynamicFormsSerializer(UUIDMixIn, ActionMixin, RenderToTableMixin):
         Returns serializer for filter row in table
         :return:  Serializer
         """
-        if not getattr(type(self), '_filter_data', None):
-            _filter_data = type(self)(instance=type(self).Meta.model())
-            _filter_data.serializer_type = 'filter'
-            for name, field in _filter_data.fields.fields.items():
-                # if isinstance(field, fields.ChoiceField):
-                #     field.allow_blank = True
-                # elif isinstance(field, fields.IntegerField):
-                # TODO: this doesn't work yet: the IntegerField will still have its valueset to whatever the default was
-                field.allow_blank = True
-                field.allow_null = True
-                field.default = None
-                _filter_data.data[name] = None
-
-            type(self)._filter_data = _filter_data
-        return type(self)._filter_data
+        if getattr(self, '_filter_ser', None) is None:
+            # noinspection PyAttributeOutsideInit
+            self._filter_ser = type(self)(is_filter=True)
+        return self._filter_ser  # Just create the same serializer in filter mode (None values, allow_nulls)
 
     # noinspection PyUnusedLocal
     def suppress_action(self, action, request, viewset):
@@ -84,7 +99,7 @@ class DynamicFormsSerializer(UUIDMixIn, ActionMixin, RenderToTableMixin):
         return False
 
     @property
-    def renderable_actions(self):
+    def renderable_actions(self: 'serializers.Serializer'):
         """
         Returns those actions that are not suppressed
         :return: List[Action]
@@ -93,6 +108,22 @@ class DynamicFormsSerializer(UUIDMixIn, ActionMixin, RenderToTableMixin):
         viewset = self.context.get('view', None)
         return [action for action in self.controls.actions if not self.suppress_action(action, request, viewset)]
 
+    # noinspection PyUnresolvedReferences
+    def get_initial(self) -> Any:
+        if getattr(self, '_errors', None):
+            # This basically reproduces BaseSerializer.data property except that it disregards the _errors member
+            if self.instance:
+                res = self.to_representation(self.instance)
+            # elif hasattr(self, '_validated_data'):
+            #     res = self.to_representation(self.validated_data)
+            else:
+                res = {}
+            res.update(super().get_initial())
+
+            return res
+
+        return super().get_initial()
+
 
 class ModelSerializer(DynamicFormsSerializer, serializers.ModelSerializer):
     """
@@ -119,6 +150,10 @@ class ModelSerializer(DynamicFormsSerializer, serializers.ModelSerializer):
     the serializer class, or simply use a `Serializer` class.
     """
 
+    def __init__(self, *args, is_filter: bool = False, **kwds):
+        super().__init__(*args, is_filter=is_filter, **kwds)
+        self.manage_changed_flds()
+
     serializer_field_mapping = {
         models.AutoField: fields.IntegerField,
         models.BigIntegerField: fields.IntegerField,
@@ -153,7 +188,31 @@ class ModelSerializer(DynamicFormsSerializer, serializers.ModelSerializer):
     serializer_url_field = fields.HyperlinkedIdentityField
     serializer_choice_field = fields.ChoiceField
 
+    def manage_changed_flds(self):
+        """
+        When there is a need to only change few parameters of a field put those fields and changed parameters in
+        serializers Meta class in parameter changed_flds.
+
+        Exsample:
+        changed_flds = {'id': dict(display=DisplayMode.HIDDEN),
+                        'comment': dict(label='Comm', help_text='Help text for comment field')
+                        }
+        :return:
+        """
+        if hasattr(self.Meta, 'changed_flds'):
+            for field, params in self.Meta.changed_flds.items():
+                field_def = self.fields.get(field, None)
+                if field_def:
+                    for key, val in params.items():
+                        setattr(field_def, key, val)
+
 
-# noinspection PyAbstractClass
 class Serializer(DynamicFormsSerializer, serializers.Serializer):
-    pass
+
+    def update(self, instance, validated_data):
+        # Implemented just so IDE doesn't complain. Normally this will be handled in SingleRecordViewSet
+        pass
+
+    def create(self, validated_data):
+        # Implemented just so IDE doesn't complain. Normally this will be handled in SingleRecordViewSet
+        pass
