@@ -43,9 +43,10 @@ TLD.prototype = {
 dynamicforms = {
   // DYNAMICFORMS is an object containing all dynamicforms settings as specified by defaults and in settings.py
   DYNAMICFORMS: {
-    'template': 'dynamicforms/bootstrap/',
-    'jquery_ui': false,
-    'edit_in_dialog': true,
+    'template':          'dynamicforms/bootstrap/',
+    'jquery_ui':         false,
+    'edit_in_dialog':    true,
+    'bootstrap_version': 'v4',
   },
 
   /**
@@ -54,11 +55,163 @@ dynamicforms = {
    * @param status
    * @param error
    */
-  showAjaxError: function showAjaxError(xhr, status, error) {
+  showAjaxError:   function showAjaxError(xhr, status, error) {
     //TODO: make proper error display message. You will probably also need some text about what you were trying to do
     console.log([xhr, status, error]);
   },
+  /**
+   * Shows progress dialog if operation takes longer than 0.5 seconds. Also sets z-index of progress bar and overlay
+   * @param progressDlgID: ID of progress dialog element - for custom progress dialogs.
+   * @param progressSettings: Progress dialog settings - for custom progress dialogs.
+   */
+  showProgressDlg: function showProgressDialog(progressDlgID, progressSettings) {
+    var zIndexElement = null;
+    var progressDlg   = $('#' + progressDlgID);
+    if (dynamicforms.DYNAMICFORMS.jquery_ui) {
+      if (progressSettings === undefined)
+        progressSettings = {value: 25};
+      $("#df-progress-bar-indeterminate").progressbar(progressSettings);
+      progressSettings['value'] = 0;
+      $("#df-progress-bar-determinate").progressbar(progressSettings);
 
+      if (dynamicforms.shouldShowProgressDlg) {
+        dynamicforms.progressDlgShown = true;
+        progressDlg.dialog({dialogClass: 'progress-bar', closeOnEscape: false, resizable: false});
+      }
+      zIndexElement = progressDlg.parents().first();
+    } else {
+      if (progressSettings === undefined)
+        progressSettings = {keyboard: false, backdrop: 'static'};
+      if (dynamicforms.shouldShowProgressDlg) {
+        dynamicforms.progressDlgShown = true;
+        progressDlg.modal(progressSettings);
+      }
+      zIndexElement = progressDlg;
+    }
+    var zIndexOrig = zIndexElement.css('z-index');
+    var zIndex     = parseInt(zIndexOrig);
+    if (zIndex == NaN)
+      zIndex = zIndexOrig;
+    else {
+      // Because all bootstrap modules have same z-index. If we want that clickable progress bar will be on top of all
+      // we must raise z-index. At the same time we must set overlay z-index, so nothing else will be clickable.
+      zIndex += 1;
+      zIndexElement.css('z-index', zIndex + 1);
+    }
+
+    $('#df-overlay').css('z-index', zIndex);
+  },
+
+  progressCheckInterval: null,
+  /**
+   * Sets continuous checking on server about operation progress. After first check shows progress dialog if necessary.
+   * @param progressDlgID: ID of progress dialog element - for custom progress dialogs.
+   * @param timestamp: used for generating operation progress key under which progress is stored on server.
+   * @param progressSettings: Progress dialog settings - for custom progress dialogs.
+   */
+  progressCheck:         function progressCheck(progressDlgID, timestamp, progressSettings) {
+    if (!dynamicforms.progressDlgShown && dynamicforms.progressCheckInterval != null) {
+      clearInterval(dynamicforms.progressCheckInterval);
+      dynamicforms.progressCheckInterval = null;
+    } else {
+      $.ajax({url: '/dynamicforms/progress/', headers: {'X-DF-TIMESTAMP': timestamp}})
+        .done(function (data, textStatus, jqXHR) {
+          var pb_indet   = $('#df-progress-bar-indeterminate');
+          var pb_det     = $('#df-progress-bar-determinate');
+          var percent    = data['value'];
+          var show_indet = percent == null;
+          if (!show_indet) {
+            if (dynamicforms.DYNAMICFORMS.jquery_ui) {
+              pb_det.progressbar("value", parseInt(percent));
+            } else {
+              pb_det.css('width', percent + '%').attr('aria-valuenow', percent)
+            }
+          }
+          pb_indet.toggle(show_indet);
+          pb_det.toggle(!show_indet);
+
+          if (dynamicforms.shouldShowProgressDlg && !dynamicforms.progressDlgShown) {
+            dynamicforms.showProgressDlg(progressDlgID, progressSettings);
+          }
+        })
+        .fail(function (jqXHR, textStatus, errorThrown) {
+          if (dynamicforms.shouldShowProgressDlg && !dynamicforms.progressDlgShown) {
+            dynamicforms.showProgressDlg(progressDlgID, progressSettings);
+          }
+        });
+    }
+  },
+  /**
+   * Starts continuous checking on operations progress. After first check progress dialog is opened if necessary
+   * @param progressDlgID: ID of progress dialog element - for custom progress dialogs.
+   * @param timestamp: used for generating operation progress key under which progress is stored on server.
+   * @param progressSettings: Progress dialog settings - for custom progress dialogs.
+   */
+  startProgressChecker:  function startProgressChecker(progressDlgID, timestamp, progressSettings) {
+    dynamicforms.progressCheck(progressDlgID, timestamp, progressSettings);
+    dynamicforms.progressCheckInterval = setInterval(dynamicforms.progressCheck, 500, progressDlgID, timestamp, progressSettings);
+  },
+  shouldShowProgressDlg: false,
+  progressDlgShown:      false,
+  /**
+   * Sets overlay so nothing can be clicked while operation last. If it last for more than 0.5 second it starts progress checker.
+   * @param progressDlgID: ID of progress dialog element - for custom progress dialogs.
+   * @param timestamp: used for generating operation progress key under which progress is stored on server.
+   * @param progressSettings: Progress dialog settings - for custom progress dialogs.
+   */
+  setProgressDlg:        function setProgressDlg(progressDlgID, timestamp, progressSettings) {
+    $('#df-overlay').css('z-index', 10000).show();
+    dynamicforms.shouldShowProgressDlg = true;
+    window.setTimeout(function () {
+      if (dynamicforms.shouldShowProgressDlg) {
+        dynamicforms.startProgressChecker(progressDlgID, timestamp, progressSettings);
+      }
+    }, 500);
+  },
+  /**
+   * Closes progress dialog (if it is opened) and hides overlay that prevents clicks on other elements
+   * @param progressDlgID: ID of progress dialog element - for custom progress dialogs.
+   */
+  closeProgressDlg:      function closeProgressDlg(progressDlgID) {
+    dynamicforms.shouldShowProgressDlg = false
+    if (dynamicforms.progressCheckInterval != null) {
+      clearInterval(dynamicforms.progressCheckInterval);
+      dynamicforms.progressCheckInterval = null;
+    }
+
+    $('#df-overlay').hide();
+    if (dynamicforms.progressDlgShown) {
+      dynamicforms.progressDlgShown = false;
+      if (dynamicforms.DYNAMICFORMS.jquery_ui) {
+        $('#' + progressDlgID).dialog('close');
+      } else {
+        $('#' + progressDlgID).modal('hide');
+      }
+    }
+  },
+  /**
+   * Calls standard jQuery.ajax. Additionally it sets overlay that prevents clicks on other elements until operation completes
+   * If operation lasts for more than 0.5 seconds progress dialog is shown.
+   * @param options: Dict with options for ajax call ('ajax_setts'), and custom progress dialog ('progress_id', 'progress_sets')
+   * @returns ajax promise with everything set for progress dialog: X-DF-TIMESTAMP header for progress checking and
+   *  callbacks for closing progress dialog after operation completes
+   */
+  ajaxWithProgress:      function ajaxWithProgress(options) {
+    var progressDlgID = options['progress_id'] !== undefined ? options['progress_id'] : 'df-progress-bar-container';
+    var timestamp     = $.now()
+
+    dynamicforms.setProgressDlg(progressDlgID, timestamp, options['progress_setts']);
+    var closeProgressDialogFunc = function () {
+      dynamicforms.closeProgressDlg(progressDlgID);
+    };
+
+    var ajaxSettings          = options['ajax_setts'] !== undefined ? options['ajax_setts'] : {};
+    var headers               = ajaxSettings['headers'] !== undefined ? ajaxSettings['headers'] : {};
+    headers['X-DF-TIMESTAMP'] = timestamp;
+    ajaxSettings['headers'] = headers;
+
+    return $.ajax(ajaxSettings).done(closeProgressDialogFunc).fail(closeProgressDialogFunc);
+  },
 
   /**
    * Handles what happens when user says "Save data". Basically serialization, send to server, response to returned
@@ -75,62 +228,67 @@ dynamicforms = {
 
     headers['X-CSRFToken'] = dynamicforms.csrf_token;
 
-    var recordURL = dynamicforms.getRecordURL();
+    var recordURL = dynamicforms.getRecordURL($form.attr('id'));
     var recordID  = data.id ? data.id : false;
 
     var listId = dynamicforms.form_helpers.get($form.attr('id'), 'listID');
 
     var dataType = 'html';
     if (doneFunc == undefined) {
-      doneFunc = function(data) {
+      doneFunc = function (data) {
         //var formContent = $dlg.find("form").html();
         dynamicforms.closeDialog($dlg);
+        if (!recordID) {
+          try {
+            recordID = $(data).find('form.dynamicforms-form').find('input[name=\'id\']').val().trim();
+            if (recordID == '')
+              recordID = false;
+          } catch (e) {}
+        }
         dynamicforms.refreshList(recordURL, recordID, refreshType, listId);
       }
-    }
-    else
+    } else
       dataType = 'json';  // This is a brazen assumption that custom done functions will only ever work with JSON
 
-    $.ajax({
-      type:        method,
-      url:         $form.attr("action"),
-      data:        data,
-      dataType:    dataType,
-      headers:     headers,
-      traditional: true,
-    })
+    dynamicforms.ajaxWithProgress({
+                                    ajax_setts: {
+                                      type:        method,
+                                      url:         $form.attr("action"),
+                                      data:        data,
+                                      dataType:    dataType,
+                                      headers:     headers,
+                                      traditional: true,
+                                    }
+                                  })
       .done(doneFunc)
       .fail(function (xhr, status, error) {
         // TODO: this doesn't handle errors correctly: if return status is 400 something, it *might* be OK
-        // but if it's 500 something, dialog will be replaced by non-dialog code and displaying it will fail
-        // also for any authorization errors, CSRF, etc, it will again fail
-        // Try finding a <div class="dynamicforms-dialog"/> in there to see if you actually got a dialog
-        dynamicforms.updateDialog($dlg, $(xhr.responseText));
+        //  but if it's 500 something, dialog will be replaced by non-dialog code and displaying it will fail
+        //  also for any authorization errors, CSRF, etc, it will again fail
+        //  Try finding a <div class="dynamicforms-dialog"/> in there to see if you actually got a dialog
+        dynamicforms.updateDialog($dlg, $(xhr.responseText), refreshType, listId);
       });
   },
 
-  /**
-   * Holds record specific url
-   */
-  recordURL: '',
-
   /**
    * Sets record specific url
+   * @param formID: id of form
    * @param url: url for record
    */
-  setRecordURL: function setRecordURL(url) {
-    dynamicforms.recordURL = url;
+  setRecordURL: function setRecordURL(formID, url) {
+    dynamicforms.form_helpers.set(formID, 'recordURL', url);
   },
 
   /**
    * Gets record specific url
+   * @param formID: id of form
    */
-  getRecordURL: function getRecordURL() {
-    return dynamicforms.recordURL;
+  getRecordURL: function getRecordURL(formID) {
+    return dynamicforms.form_helpers.get(formID, 'recordURL');
   },
 
   isFunction: function isFunction(passedFunction) {
-    return typeof(eval(passedFunction) === "function") ? true : false;
+    return typeof (eval(passedFunction)) == "function";
   },
 
   /**
@@ -143,13 +301,25 @@ dynamicforms = {
    */
   refreshList: function refreshList(url, recordID, refreshType, formID, deletion) {
     if (refreshType == undefined || refreshType == 'record') {
-      $.ajax({type: 'GET', url: url, dataType: 'html'})
-        .done(function (data) {
-          dynamicforms.refreshRow(data, formID, recordID);
-        })
-        .fail(function (xhr, status, error) {
-          // TODO: this doesn't handle errors correctly
-        });
+      if (recordID) {
+        var trSelector = "tr[data-id='" + recordID + "']";
+        $rowToRefresh  = $(trSelector); // Row to refresh
+      }
+
+      // If table will not change we wont even fetch new record
+      if (!dynamicforms.isLinkNext(dynamicforms.df_tbl_pagination.get(formID, undefined).link_next) ||
+        ($rowToRefresh != null && $rowToRefresh.length)) {
+        var data = dynamicforms.filterData(formID, true);
+        if (recordID)
+          data.id = recordID
+        dynamicforms.ajaxWithProgress({ajax_setts: {type: 'GET', url: url, data: data, dataType: 'html'}})
+          .done(function (data) {
+            dynamicforms.refreshRow(data, formID, recordID);
+          })
+          .fail(function (xhr, status, error) {
+            // TODO: this doesn't handle errors correctly
+          });
+      }
     } else if (refreshType == 'table') {
       dynamicforms.refreshTable(formID, recordID);
       if (deletion == true) {
@@ -157,52 +327,68 @@ dynamicforms = {
       }
     } else if (refreshType == 'page') {
       window.location.reload(true);
-    } else if (refreshType.indexOf('redirect') !== -1) {
-      var redirectUrl = refreshType.split(':').pop();
-      window.location.href = redirectUrl;
+    } else if (typeof (refreshType) == 'function') {
+      refreshType();
+    } else if (refreshType.indexOf('redirect:') !== -1) {
+      window.location.href = refreshType.split(':').pop();
     } else if (refreshType == 'no refresh') {
       // pass
-    } else if (dynamicforms.isFunction(refreshType)){
+    } else if (dynamicforms.isFunction(refreshType)) {
       // Change passed string to function call
       var functionString = refreshType + "();";
-      var customFunction = new Function (functionString);
+      var customFunction = new Function(functionString);
       customFunction();
     }
   },
-
+  /**
+   * Insert new row after the last row or insert first row
+   * @param $newRow
+   */
+  insertRow:   function insertRow($newRow) {
+    // Insert new row after the last row or insert first row
+    if ($newRow.length) {
+      var $lastRow = $("tr[data-id]").last(); // Last row before adding new record
+      if ($lastRow.length) {
+        $newRow.insertAfter($lastRow);
+      } else {
+        $("table").find("tr[data-title]").replaceWith($newRow);
+      }
+    }
+  },
   /**
    * Replaces edited row
    * @param data
    * @param recordID: id of edited data
    * @param formID: id of form
    */
-  refreshRow: function refreshRow(data, formID, recordID) {
+  refreshRow:  function refreshRow(data, formID, recordID) {
     var tbl_pagination = dynamicforms.df_tbl_pagination.get(formID, undefined);
-    var link_next      = dynamicforms.form_helpers.get(formID, 'reverseRowURL');
 
-    // Case when table is smaller than pagination
-    if (link_next == undefined) {
+    var $rowToRefresh = null; // Row to refresh
+
+    if (recordID) {
+      var trSelector = "tr[data-id='" + recordID + "']";
+      $rowToRefresh  = $(trSelector); // Row to refresh
+    }
+
+    // Case when all records are loaded or edited record is loaded
+    if (!dynamicforms.isLinkNext(tbl_pagination.link_next) || ($rowToRefresh != null && $rowToRefresh.length)) {
       var $htmlObject = $(data);
 
       if (recordID) {
-        var trSelector    = "tr[data-id='" + recordID + "']";
-        var $editedRow    = $htmlObject.find(trSelector); // Edited record from ajax returned html
-        var $rowToRefresh = $(trSelector); // Row to refresh
-        $rowToRefresh.replaceWith($editedRow);
-      } else {
-        var $lastRow = $("tr[data-id]").last(); // Last row before adding new record
-        var $newRow  = $htmlObject.find("table").find("tr[data-id]").last(); // Added record from ajax returned html
-
-        // Insert new row after the last row or insert first row
-        if ($lastRow.length) {
-          $newRow.insertAfter($lastRow);
-        } else {
-          $("table").find("tr[data-title]").replaceWith($newRow);
+        var $editedRow = $htmlObject.find(trSelector); // Edited record from ajax returned html
+        if ($editedRow.length) {
+          if ($rowToRefresh.length)
+            $rowToRefresh.replaceWith($editedRow);
+          else
+            dynamicforms.insertRow($editedRow);
         }
+      } else {
+        var $newRow = $htmlObject.find("table").find("tr[data-id]").last(); // Added record from ajax returned html
+        dynamicforms.insertRow($newRow);
       }
-    } else { // Case when table is larger than pagination
-      dynamicforms.filterData(formID);
     }
+    // else do nothing. Paginator will take care of new record when necessary
   },
 
   /**
@@ -211,48 +397,10 @@ dynamicforms = {
    * @param recordID: id of edited data
    */
   refreshTable: function refreshTable(formID, recordID) {
-    var tbl_pagination = dynamicforms.df_tbl_pagination.get(formID, undefined);
-    var link_next      = dynamicforms.form_helpers.get(formID, 'reverseRowURL');
 
-    var recordURL = dynamicforms.getRecordURL();
-    var table     = $("#list-" + formID).find("tbody:first");
-
-    // Case when table is smaller than pagination
-    if (link_next == undefined) {
-      $.ajax({
-        type:    'GET',
-        headers: {'X-CSRFToken': dynamicforms.csrf_token, 'X-DF-RENDER-TYPE': 'table rows'},
-        url:     recordURL,
-      }).done(function (data) {
-
-        data                     = $(data).filter("tr");
-        tbl_pagination.link_next = data[0].getAttribute('data-next');
-
-        // Edited record
-        if (recordID) {
-          table.find("tr").remove();
-        } else if (!recordID) { // Added record
-          // Check if "no data" element is present and remove it
-          table.find("tr[data-title='NoData']").remove();
-          // Update records
-          for (var i = data.length - 1; i >= 0; i--) {
-            var data_id = data[i].getAttribute('data-id');
-            if (data_id == null || table.find("tr[data-id='" + data_id + "']").length > 0)
-              data.splice(i, 1);
-          }
-        }
-
-        if (data.length > 0) {
-          table.append(data);
-          tbl_pagination.trigger_element = data[0];
-        }
-        dynamicforms.paginatorCheckGetNextPage(formID);
-      }).fail(function (xhr, status, error) {
-        // TODO: what if the server returns an error? Do we continue with pagination? (Task #100)
-      });
-    } else { // Case when table is larger than pagination
-      dynamicforms.filterData(formID);
-    }
+    //If it is set to refresh table, we also want to see all the changes, that was made by other users.
+    //We can only get them by reread data.
+    dynamicforms.filterData(formID);
   },
 
   /**
@@ -282,15 +430,29 @@ dynamicforms = {
       dynamicforms.submitForm($dlg, $form, refreshType);
     });
     // And show the dialog
-    (dynamicforms.DYNAMICFORMS.jquery_ui) ? $dlg.dialog('open') : $dlg.modal();
+    if (dynamicforms.DYNAMICFORMS.jquery_ui)
+      $dlg.dialog('open')
+    else {
+      $dlg.on('hidden.bs.modal', function () {
+        $dlg.remove();
+        if ($dlg.showNewAfterHide)
+          dynamicforms.showDialog($dlg.showNewAfterHide, refreshType, listId);
+      });
+      $dlg.modal();
+    }
   },
 
   /**
    * Updates the current dialog with errors.
    * @param $dlg: dialog to update
    * @param $newDlg: new dialog with errors in form
+   * @param refreshType: how to refresh the table after the dialog is finished with editing
+   * @param listId: id of table element with records
    */
-  updateDialog: function updateDialog($dlg, $newDlg) {
+  updateDialog: function updateDialog($dlg, $newDlg, refreshType, listId) {
+    $dlg.showNewAfterHide = $newDlg;  // Old dialog will show the new one after being hidden
+    dynamicforms.closeDialog($dlg, refreshType, listId);
+    /*
     // Replace current form with new form containing errors
     var newForm     = $newDlg.find("form");
     var currentForm = $dlg.find("form");
@@ -300,15 +462,25 @@ dynamicforms = {
     var dlgId       = $dlg.attr("id").split("-").slice(1).join("-");
     var updatedForm = $dlg.find("form");
     updatedForm.attr("id", dlgId);
+    */
   },
 
   /**
    * Closes the current dialog
    * TODO: adjust hashURL
    * @param $dlg: dialog to close
+   * @param refreshType: how to refresh the table after the dialog is finished with editing
+   * @param listId: id of table element with records
    */
-  closeDialog: function closeDialog($dlg) {
-    (dynamicforms.DYNAMICFORMS.jquery_ui) ? $dlg.remove() : $dlg.modal('hide');
+  closeDialog: function closeDialog($dlg, refreshType, listId) {
+    if (!dynamicforms.DYNAMICFORMS.jquery_ui) {
+      $dlg.modal('hide');
+    } else {
+      $dlg.remove();
+      if ($dlg.showNewAfterHide) {
+        dynamicforms.showDialog($dlg.showNewAfterHide, refreshType, listId);
+      }
+    }
   },
 
   /**
@@ -319,11 +491,12 @@ dynamicforms = {
    */
   editRow: function editRow(recordURL, refreshType, listId) {
     if (dynamicforms.DYNAMICFORMS.edit_in_dialog) {
-      recordURL += '?df_render_type=dialog'; // TODO: is this necessary? we already add the header
-      $.ajax({
-        url:     recordURL,
-        headers: {'X-DF-RENDER-TYPE': 'dialog'},
-      })
+      dynamicforms.ajaxWithProgress({
+                                      ajax_setts: {
+                                        url:     recordURL,
+                                        headers: {'X-DF-RENDER-TYPE': 'dialog'},
+                                      }
+                                    })
         .done(function (dialogHTML) {
           dynamicforms.showDialog($(dialogHTML), refreshType, listId);
         })
@@ -368,11 +541,13 @@ dynamicforms = {
    */
   deleteRow: function deleteRow(recordURL, recordID, refreshType, listId) {
     //TODO: Ask user for confirmation
-    $.ajax({
-      url:     recordURL,
-      method:  'DELETE',
-      headers: {'X-CSRFToken': dynamicforms.csrf_token},
-    })
+    dynamicforms.ajaxWithProgress({
+                                    ajax_setts: {
+                                      url:     recordURL,
+                                      method:  'DELETE',
+                                      headers: {'X-CSRFToken': dynamicforms.csrf_token},
+                                    }
+                                  })
       .done(function (dialogHTML) {
         console.log('Record successfully deleted.');
         //  TODO: make a proper notification
@@ -380,20 +555,22 @@ dynamicforms = {
         if (refreshType == undefined || refreshType == 'record') {
           dynamicforms.removeRow(recordID);
         } else if (refreshType == 'table') {
-          var recordURL = dynamicforms.getRecordURL();
+          var recordURL = dynamicforms.getRecordURL(listId);
           // var formId    = $('table')[0].getAttribute('id').replace('list-', '');
           dynamicforms.refreshList(recordURL, true, refreshType, listId, true);
         } else if (refreshType == 'page') {
           window.location.reload(true);
+        } else if (typeof (refreshType) == 'function') {
+          refreshType();
         } else if (refreshType.indexOf('redirect') !== -1) {
-          var redirectUrl = refreshType.split(':').pop();
+          var redirectUrl      = refreshType.split(':').pop();
           window.location.href = redirectUrl;
         } else if (refreshType == 'no refresh') {
           // pass
-        } else if (dynamicforms.isFunction(refreshType)){
+        } else if (dynamicforms.isFunction(refreshType)) {
           // Change passed string to function call
           var functionString = refreshType + "();";
-          var customFunction = new Function (functionString);
+          var customFunction = new Function(functionString);
           customFunction();
         }
       })
@@ -404,7 +581,6 @@ dynamicforms = {
   },
 
 
-
   /**
    * Handles what should happen when user clicks "Add new" button
    * Right now newRow doesn't do anything distinct, so let's just call editRow
@@ -467,9 +643,9 @@ dynamicforms = {
   removeFormDeclarations: function removeFormDeclarations($form) {
     var formID = $form.attr('id');
     $.each(dynamicforms.form_helpers.getOrCreate(formID, 'fields', {}),
-      function (fieldID) {
-        dynamicforms.field_helpers.del(fieldID);
-      }
+           function (fieldID) {
+             dynamicforms.field_helpers.del(fieldID);
+           }
     );
     dynamicforms.form_helpers.del(formID);
   },
@@ -658,11 +834,21 @@ dynamicforms = {
    *
    * @param field: id or jQuery object of the field
    * @param value: new value to set
+   * @param select2_ajax_option_text: label of selected option to be added to select2 options list. Only use when select2 is ajax
    */
-  fieldSetValue: function fieldSetValue(field, value) {
+  fieldSetValue: function fieldSetValue(field, value, select2_ajax_option_text) {
     var $field = field instanceof jQuery ? field : dynamicforms.field_helpers.get(field, '$field');
     if ($field.attr('type') == 'checkbox')
       return $field.prop('checked', value);
+    if ($field.data('select2')) {
+      if ($field.data('select2').options.options.ajax) {
+        var opt = $('<option value="' + value + '"></option>').text(select2_ajax_option_text);
+        $field.append(opt);
+      }
+      $field.val(value);
+      $field.trigger('change');
+      return;
+    }
     $field.val(value);
   },
 
@@ -674,9 +860,9 @@ dynamicforms = {
    */
   fieldSetVisible: function fieldSetVisible(field, visible) {
     //TODO: we need to check parent container if everything inside it is hidden. If there is, the parent container needs to hide too
-    var $field  = field instanceof jQuery ? field : dynamicforms.field_helpers.get(field, '$field');
-    var fieldID = $field.attr('id');
-    var $hide = $field.parents('#container-' + fieldID);
+    var $field      = field instanceof jQuery ? field : dynamicforms.field_helpers.get(field, '$field');
+    var fieldID     = $field.attr('id');
+    var $hide       = $field.parents('#container-' + fieldID);
     var $hideParent = $hide.parents('[data-hide-with-field]');
     if ($hideParent.length > 0)
       $hide = $hideParent;
@@ -716,7 +902,7 @@ dynamicforms = {
    */
   paginatorInitTable: function paginatorInitTable(formID, link_next, link_prev) {
     dynamicforms.df_tbl_pagination.set(formID, 'link_next', link_next);
-    if (link_next != '' && link_next != undefined) {
+    if (dynamicforms.isLinkNext(link_next)) {
       var table_rows = $("#list-" + formID).find("tbody:first").find("tr");
       dynamicforms.df_tbl_pagination.set(formID, 'trigger_element', table_rows[0]);
     }
@@ -763,6 +949,8 @@ dynamicforms = {
     var link_next      = '';
     if (filter.length) {
       link_next = dynamicforms.form_helpers.get(formID, 'reverseRowURL');
+      if (link_next == undefined)
+        link_next = window.location.origin + window.location.pathname
       if (filter != 'nofilter')
         link_next += '?' + filter;
     } else
@@ -772,7 +960,7 @@ dynamicforms = {
     console.log(tbl_pagination.last_link_next);
     console.log(filter.length);*/
 
-    if (link_next != null && link_next != "None" && (link_next != tbl_pagination.last_link_next || filter.length)) {
+    if (dynamicforms.isLinkNext(link_next) && (link_next != tbl_pagination.last_link_next || filter.length)) {
       tbl_pagination.last_link_next = link_next;
 
       var table = $("#list-" + formID).find("tbody:first");
@@ -783,10 +971,10 @@ dynamicforms = {
       $("#loading-" + formID).show();
       //TODO: Remember sequence number... if data that comes back has other than last sequence number than just ignore it #114
       $.ajax({
-        type:    'GET',
-        headers: {'X-CSRFToken': dynamicforms.csrf_token, 'X-DF-RENDER-TYPE': 'table rows'},
-        url:     link_next,
-      }).done(function (data) {
+               type:    'GET',
+               headers: {'X-CSRFToken': dynamicforms.csrf_token, 'X-DF-RENDER-TYPE': 'table rows'},
+               url:     link_next,
+             }).done(function (data) {
 
         data                     = $(data).filter("tr");
         tbl_pagination.link_next = data[0].getAttribute('data-next');
@@ -807,6 +995,9 @@ dynamicforms = {
         if (data.length > 0) {
           table.append(data);
           tbl_pagination.trigger_element = data[0];
+          if (table.find("tr[data-title=NoData]").length > 0) {
+            table.find("tr[data-title=NoData]").remove()
+          }
         }
         dynamicforms.paginatorCheckGetNextPage(formID);
       }).fail(function (xhr, status, error) {
@@ -844,8 +1035,11 @@ dynamicforms = {
    * Prepares filter string and calls server to get filtered data
    *
    * @param formID: id of table object
+   * @param returnDict: set to true if only filter data should be returned
    */
-  filterData: function filterData(formID) {
+  filterData: function filterData(formID, returnDict) {
+    if (returnDict == undefined)
+      returnDict = false;
     var filter = {};
     $("#list-" + formID).find(".dynamicforms-filterrow th").each(function (index) {
 
@@ -856,9 +1050,18 @@ dynamicforms = {
           filter[element.attr("name")] = true;
         else if (!element.is('[readonly]'))
           filter[element.attr("name")] = false;
-      } else if (element.val() != null && element.val().length)
-        filter[element.attr("name")] = element.val();
+      } else if (element.val() != null && element.val().length) {
+        if (element.is('select') && element.is("[multiple]")) {
+          var delimiter = element.attr('value_delimiter');
+          if (delimiter == undefined)
+            delimiter = ',';
+          filter[element.attr("name")] = element.val().join(delimiter);
+        } else
+          filter[element.attr("name")] = element.val();
+      }
     });
+    if (returnDict)
+      return filter;
     filter = jQuery.param(filter);
     if (!filter.length)
       filter = 'nofilter';
@@ -879,6 +1082,22 @@ dynamicforms = {
     dynamicforms.filterData(formId);
   },
 
+  isLinkNext: function isLinkNext(link_next) {
+    return link_next != null && link_next != undefined && link_next != '' && link_next != "None";
+  },
+
+  select2Opening: function select2Opening(evt, $select2, fnc) {
+    if ($select2.data('unselecting')) {
+      $select2.removeData('unselecting');
+      evt.preventDefault();
+    } else if (fnc != null)
+      fnc(evt, $select2);
+  },
+
+  select2Unselecting: function select2Unselecting($select2) {
+    $select2.data('unselecting', true);
+  }
+
 };
 
 $(document).ready(function () {
@@ -889,4 +1108,8 @@ $(document).ready(function () {
     dynamicforms.serializeForm($(form), 'final');
   });
   window.setInterval(dynamicforms.paginatorCheckGetNextPageAll, 100);
+
+  var $overlay = $("<div id='df-overlay' style='position: fixed; display: none; width: 100%; height: 100%; top: 0; " +
+                     "left: 0; right: 0; bottom: 0; cursor: pointer; z-index: auto'></div>");
+  $("body").append($overlay);
 })
